window.Gleam.initSearch([{"doc":"lustre","title":"lustre","content":"# Lustre\n\nA framework for building create web apps – powered by Gleam and React!\n\n---\n\n[![Package Version](https://img.shields.io/hexpm/v/lustre)](https://hex.pm/packages/lustre)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/lustre/)\n\n```gleam\nimport gleam/int\nimport lustre\nimport lustre/element.{button, div, p, text}\nimport lustre/event.{on_click}\nimport lustre/cmd\n\npub fn main() {\n  let app = lustre.application(#(0, cmd.none()), update, render)\n  lustre.start(app, &quot;#app&quot;)\n}\n\npub type Action {\n  Incr\n  Decr\n}\n\nfn update(state, action) {\n  case action {\n    Incr -&gt; #(state + 1, cmd.none())\n    Decr -&gt; #(state - 1, cmd.none())\n  }\n}\n\nfn render(state) {\n  div(\n    [],\n    [\n      button([on_click(Decr)], [text(&quot;-&quot;)]),\n      p([], [text(int.to_string(state))]),\n      button([on_click(Incr)], [text(&quot;+&quot;)]),\n    ],\n  )\n}\n```\n\n---\n\n❗️ This package relies on Gleam&#39;s JavaScript FFI and is intended to be run in\nthe browser. **It will not work if your are targetting Node.js or Erlang.**\n\n---\n\n## Installation\n\nIf available on Hex, this package can be added to your Gleam project:\n\n```sh\ngleam add lustre\n```\n\nand its documentation can be found at &lt;https://hexdocs.pm/lustre&gt;. You will also\nneed to install `react` and `react-dom` from npm:\n\n```sh\nnpm i react react-dom\n```\n\n---\n\n## Development\n\nFirst, make sure you have both Gleam and Node.js installed, then:\n\n```bash\nnpm i\nnpm start\n```\n\nThis sets up `chokidar` to watch our gleam source code and runs the compiler\nwhenever we make a change. It also starts a server that will serve the examples\nlocated in `test/example/`.\n","url":"index.html"},{"doc":"lustre/attribute","title":"Attribute","content":"pub opaque type Attribute(msg)\n Attributes are attached to specific elements. They&#39;re either key/value pairs\n or event handlers.\n\n\nSynonyms:\nlustre/attribute.Attribute\nlustre/attribute Attribute","url":"lustre/attribute.html#Attribute"},{"doc":"lustre/attribute","title":"accept","content":"pub fn accept(types: List(String)) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.accept\nlustre/attribute accept","url":"lustre/attribute.html#accept"},{"doc":"lustre/attribute","title":"accept_charset","content":"pub fn accept_charset(types: List(String)) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.accept_charset\nlustre/attribute accept_charset","url":"lustre/attribute.html#accept_charset"},{"doc":"lustre/attribute","title":"alt","content":"pub fn alt(text: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.alt\nlustre/attribute alt","url":"lustre/attribute.html#alt"},{"doc":"lustre/attribute","title":"attribute","content":"pub fn attribute(name: String, value: a) -&gt; Attribute(b)\n\nSynonyms:\nlustre/attribute.attribute\nlustre/attribute attribute","url":"lustre/attribute.html#attribute"},{"doc":"lustre/attribute","title":"autocomplete","content":"pub fn autocomplete(should_autocomplete: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.autocomplete\nlustre/attribute autocomplete","url":"lustre/attribute.html#autocomplete"},{"doc":"lustre/attribute","title":"autofocus","content":"pub fn autofocus(should_autofocus: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.autofocus\nlustre/attribute autofocus","url":"lustre/attribute.html#autofocus"},{"doc":"lustre/attribute","title":"autoplay","content":"pub fn autoplay(should_autoplay: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.autoplay\nlustre/attribute autoplay","url":"lustre/attribute.html#autoplay"},{"doc":"lustre/attribute","title":"checked","content":"pub fn checked(is_checked: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.checked\nlustre/attribute checked","url":"lustre/attribute.html#checked"},{"doc":"lustre/attribute","title":"class","content":"pub fn class(name: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.class\nlustre/attribute class","url":"lustre/attribute.html#class"},{"doc":"lustre/attribute","title":"classes","content":"pub fn classes(names: List(#(String, Bool))) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.classes\nlustre/attribute classes","url":"lustre/attribute.html#classes"},{"doc":"lustre/attribute","title":"cols","content":"pub fn cols(val: Int) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.cols\nlustre/attribute cols","url":"lustre/attribute.html#cols"},{"doc":"lustre/attribute","title":"controls","content":"pub fn controls(visible: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.controls\nlustre/attribute controls","url":"lustre/attribute.html#controls"},{"doc":"lustre/attribute","title":"disabled","content":"pub fn disabled(is_disabled: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.disabled\nlustre/attribute disabled","url":"lustre/attribute.html#disabled"},{"doc":"lustre/attribute","title":"download","content":"pub fn download(filename: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.download\nlustre/attribute download","url":"lustre/attribute.html#download"},{"doc":"lustre/attribute","title":"event","content":"pub fn event(name: String, handler: fn(Dynamic, fn(a) -&gt; Nil) -&gt;\n    Nil) -&gt; Attribute(a)\n                                                            \nSynonyms:\nlustre/attribute.event\nlustre/attribute event","url":"lustre/attribute.html#event"},{"doc":"lustre/attribute","title":"for","content":"pub fn for(id: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.for\nlustre/attribute for","url":"lustre/attribute.html#for"},{"doc":"lustre/attribute","title":"height","content":"pub fn height(val: Int) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.height\nlustre/attribute height","url":"lustre/attribute.html#height"},{"doc":"lustre/attribute","title":"href","content":"pub fn href(uri: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.href\nlustre/attribute href","url":"lustre/attribute.html#href"},{"doc":"lustre/attribute","title":"id","content":"pub fn id(name: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.id\nlustre/attribute id","url":"lustre/attribute.html#id"},{"doc":"lustre/attribute","title":"loop","content":"pub fn loop(should_loop: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.loop\nlustre/attribute loop","url":"lustre/attribute.html#loop"},{"doc":"lustre/attribute","title":"max","content":"pub fn max(val: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.max\nlustre/attribute max","url":"lustre/attribute.html#max"},{"doc":"lustre/attribute","title":"min","content":"pub fn min(val: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.min\nlustre/attribute min","url":"lustre/attribute.html#min"},{"doc":"lustre/attribute","title":"msg","content":"pub fn msg(uri: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.msg\nlustre/attribute msg","url":"lustre/attribute.html#msg"},{"doc":"lustre/attribute","title":"name","content":"pub fn name(name: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.name\nlustre/attribute name","url":"lustre/attribute.html#name"},{"doc":"lustre/attribute","title":"pattern","content":"pub fn pattern(regex: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.pattern\nlustre/attribute pattern","url":"lustre/attribute.html#pattern"},{"doc":"lustre/attribute","title":"placeholder","content":"pub fn placeholder(text: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.placeholder\nlustre/attribute placeholder","url":"lustre/attribute.html#placeholder"},{"doc":"lustre/attribute","title":"readonly","content":"pub fn readonly(is_readonly: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.readonly\nlustre/attribute readonly","url":"lustre/attribute.html#readonly"},{"doc":"lustre/attribute","title":"rel","content":"pub fn rel(relationship: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.rel\nlustre/attribute rel","url":"lustre/attribute.html#rel"},{"doc":"lustre/attribute","title":"required","content":"pub fn required(is_required: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.required\nlustre/attribute required","url":"lustre/attribute.html#required"},{"doc":"lustre/attribute","title":"rows","content":"pub fn rows(val: Int) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.rows\nlustre/attribute rows","url":"lustre/attribute.html#rows"},{"doc":"lustre/attribute","title":"selected","content":"pub fn selected(is_selected: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.selected\nlustre/attribute selected","url":"lustre/attribute.html#selected"},{"doc":"lustre/attribute","title":"src","content":"pub fn src(uri: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.src\nlustre/attribute src","url":"lustre/attribute.html#src"},{"doc":"lustre/attribute","title":"step","content":"pub fn step(val: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.step\nlustre/attribute step","url":"lustre/attribute.html#step"},{"doc":"lustre/attribute","title":"style","content":"pub fn style(properties: List(#(String, String))) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.style\nlustre/attribute style","url":"lustre/attribute.html#style"},{"doc":"lustre/attribute","title":"target","content":"pub fn target(target: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.target\nlustre/attribute target","url":"lustre/attribute.html#target"},{"doc":"lustre/attribute","title":"type_","content":"pub fn type_(name: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.type_\nlustre/attribute type_","url":"lustre/attribute.html#type_"},{"doc":"lustre/attribute","title":"value","content":"pub fn value(val: Dynamic) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.value\nlustre/attribute value","url":"lustre/attribute.html#value"},{"doc":"lustre/attribute","title":"width","content":"pub fn width(val: Int) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.width\nlustre/attribute width","url":"lustre/attribute.html#width"},{"doc":"lustre/attribute","title":"wrap","content":"pub fn wrap(mode: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.wrap\nlustre/attribute wrap","url":"lustre/attribute.html#wrap"},{"doc":"lustre/attribute","title":"lustre/attribute","content":"","url":"lustre/attribute.html"},{"doc":"lustre/element","title":"Element","content":"pub external type Element(msg)\n \n\n\nSynonyms:\nlustre/element.Element\nlustre/element Element","url":"lustre/element.html#Element"},{"doc":"lustre/element","title":"a","content":"pub fn a(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.a\nlustre/element a","url":"lustre/element.html#a"},{"doc":"lustre/element","title":"abbr","content":"pub fn abbr(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.abbr\nlustre/element abbr","url":"lustre/element.html#abbr"},{"doc":"lustre/element","title":"address","content":"pub fn address(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.address\nlustre/element address","url":"lustre/element.html#address"},{"doc":"lustre/element","title":"area","content":"pub fn area(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element.area\nlustre/element area","url":"lustre/element.html#area"},{"doc":"lustre/element","title":"article","content":"pub fn article(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.article\nlustre/element article","url":"lustre/element.html#article"},{"doc":"lustre/element","title":"aside","content":"pub fn aside(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.aside\nlustre/element aside","url":"lustre/element.html#aside"},{"doc":"lustre/element","title":"audio","content":"pub fn audio(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.audio\nlustre/element audio","url":"lustre/element.html#audio"},{"doc":"lustre/element","title":"b","content":"pub fn b(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.b\nlustre/element b","url":"lustre/element.html#b"},{"doc":"lustre/element","title":"base","content":"pub fn base(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element.base\nlustre/element base","url":"lustre/element.html#base"},{"doc":"lustre/element","title":"bdi","content":"pub fn bdi(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.bdi\nlustre/element bdi","url":"lustre/element.html#bdi"},{"doc":"lustre/element","title":"bdo","content":"pub fn bdo(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.bdo\nlustre/element bdo","url":"lustre/element.html#bdo"},{"doc":"lustre/element","title":"blockquote","content":"pub fn blockquote(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.blockquote\nlustre/element blockquote","url":"lustre/element.html#blockquote"},{"doc":"lustre/element","title":"body","content":"pub fn body(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.body\nlustre/element body","url":"lustre/element.html#body"},{"doc":"lustre/element","title":"br","content":"pub fn br(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element.br\nlustre/element br","url":"lustre/element.html#br"},{"doc":"lustre/element","title":"button","content":"pub fn button(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.button\nlustre/element button","url":"lustre/element.html#button"},{"doc":"lustre/element","title":"canvas","content":"pub fn canvas(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.canvas\nlustre/element canvas","url":"lustre/element.html#canvas"},{"doc":"lustre/element","title":"caption","content":"pub fn caption(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.caption\nlustre/element caption","url":"lustre/element.html#caption"},{"doc":"lustre/element","title":"cite","content":"pub fn cite(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.cite\nlustre/element cite","url":"lustre/element.html#cite"},{"doc":"lustre/element","title":"code","content":"pub fn code(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.code\nlustre/element code","url":"lustre/element.html#code"},{"doc":"lustre/element","title":"col","content":"pub fn col(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.col\nlustre/element col","url":"lustre/element.html#col"},{"doc":"lustre/element","title":"colgroup","content":"pub fn colgroup(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.colgroup\nlustre/element colgroup","url":"lustre/element.html#colgroup"},{"doc":"lustre/element","title":"datalist","content":"pub fn datalist(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.datalist\nlustre/element datalist","url":"lustre/element.html#datalist"},{"doc":"lustre/element","title":"dd","content":"pub fn dd(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.dd\nlustre/element dd","url":"lustre/element.html#dd"},{"doc":"lustre/element","title":"del","content":"pub fn del(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.del\nlustre/element del","url":"lustre/element.html#del"},{"doc":"lustre/element","title":"details","content":"pub fn details(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.details\nlustre/element details","url":"lustre/element.html#details"},{"doc":"lustre/element","title":"dfn","content":"pub fn dfn(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.dfn\nlustre/element dfn","url":"lustre/element.html#dfn"},{"doc":"lustre/element","title":"dialog","content":"pub fn dialog(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.dialog\nlustre/element dialog","url":"lustre/element.html#dialog"},{"doc":"lustre/element","title":"div","content":"pub fn div(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.div\nlustre/element div","url":"lustre/element.html#div"},{"doc":"lustre/element","title":"dl","content":"pub fn dl(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.dl\nlustre/element dl","url":"lustre/element.html#dl"},{"doc":"lustre/element","title":"dt","content":"pub fn dt(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.dt\nlustre/element dt","url":"lustre/element.html#dt"},{"doc":"lustre/element","title":"em","content":"pub fn em(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.em\nlustre/element em","url":"lustre/element.html#em"},{"doc":"lustre/element","title":"embed","content":"pub fn embed(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element.embed\nlustre/element embed","url":"lustre/element.html#embed"},{"doc":"lustre/element","title":"fieldset","content":"pub fn fieldset(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.fieldset\nlustre/element fieldset","url":"lustre/element.html#fieldset"},{"doc":"lustre/element","title":"figcaption","content":"pub fn figcaption(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.figcaption\nlustre/element figcaption","url":"lustre/element.html#figcaption"},{"doc":"lustre/element","title":"figure","content":"pub fn figure(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.figure\nlustre/element figure","url":"lustre/element.html#figure"},{"doc":"lustre/element","title":"footer","content":"pub fn footer(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.footer\nlustre/element footer","url":"lustre/element.html#footer"},{"doc":"lustre/element","title":"form","content":"pub fn form(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.form\nlustre/element form","url":"lustre/element.html#form"},{"doc":"lustre/element","title":"fragment","content":"pub external fn fragment(\n  children: List(Element(msg)),\n) -&gt; Element(msg)\n A fragment doesn&#39;t appear in the DOM, but allows us to treat a list of elements\n as if it were a single one. \n\nSynonyms:\nlustre/element.fragment\nlustre/element fragment","url":"lustre/element.html#fragment"},{"doc":"lustre/element","title":"h1","content":"pub fn h1(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.h1\nlustre/element h1","url":"lustre/element.html#h1"},{"doc":"lustre/element","title":"h2","content":"pub fn h2(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.h2\nlustre/element h2","url":"lustre/element.html#h2"},{"doc":"lustre/element","title":"h3","content":"pub fn h3(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.h3\nlustre/element h3","url":"lustre/element.html#h3"},{"doc":"lustre/element","title":"h4","content":"pub fn h4(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.h4\nlustre/element h4","url":"lustre/element.html#h4"},{"doc":"lustre/element","title":"h5","content":"pub fn h5(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.h5\nlustre/element h5","url":"lustre/element.html#h5"},{"doc":"lustre/element","title":"h6","content":"pub fn h6(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.h6\nlustre/element h6","url":"lustre/element.html#h6"},{"doc":"lustre/element","title":"head","content":"pub fn head(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.head\nlustre/element head","url":"lustre/element.html#head"},{"doc":"lustre/element","title":"header","content":"pub fn header(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.header\nlustre/element header","url":"lustre/element.html#header"},{"doc":"lustre/element","title":"hr","content":"pub fn hr(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element.hr\nlustre/element hr","url":"lustre/element.html#hr"},{"doc":"lustre/element","title":"html","content":"pub fn html(attrs: List(Attribute(a)), head: Element(a), body: Element(\n    a,\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.html\nlustre/element html","url":"lustre/element.html#html"},{"doc":"lustre/element","title":"i","content":"pub fn i(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.i\nlustre/element i","url":"lustre/element.html#i"},{"doc":"lustre/element","title":"iframe","content":"pub fn iframe(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element.iframe\nlustre/element iframe","url":"lustre/element.html#iframe"},{"doc":"lustre/element","title":"img","content":"pub fn img(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element.img\nlustre/element img","url":"lustre/element.html#img"},{"doc":"lustre/element","title":"input","content":"pub fn input(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element.input\nlustre/element input","url":"lustre/element.html#input"},{"doc":"lustre/element","title":"ins","content":"pub fn ins(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.ins\nlustre/element ins","url":"lustre/element.html#ins"},{"doc":"lustre/element","title":"kbd","content":"pub fn kbd(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.kbd\nlustre/element kbd","url":"lustre/element.html#kbd"},{"doc":"lustre/element","title":"label","content":"pub fn label(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.label\nlustre/element label","url":"lustre/element.html#label"},{"doc":"lustre/element","title":"legend","content":"pub fn legend(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.legend\nlustre/element legend","url":"lustre/element.html#legend"},{"doc":"lustre/element","title":"li","content":"pub fn li(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.li\nlustre/element li","url":"lustre/element.html#li"},{"doc":"lustre/element","title":"main","content":"pub fn main(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.main\nlustre/element main","url":"lustre/element.html#main"},{"doc":"lustre/element","title":"map","content":"pub external fn map(\n  element: fn() -&gt; Element(a),\n  f: fn(a) -&gt; b,\n) -&gt; Element(b)\n Transforms the actions produced by an element. Our Lustre applications have\n a single top-level `Msg` type and `update` function, and ultimately all\n elements we create must produce actions of that type (or not produce any at\n all).\n\n To save us having a monolothic `Msg` type with many constructors, we can crete\n child types and then use `map` to wrap them up in the parent type. A simple\n counter with its own message type my look like this:\n\n \n import gleam/int\n import lustre/element.{Element}\n import lustre/event\n \n type Msg {\n   Decr\n   Incr\n }\n \n pub fn update(count: Int, msg: CounterMsg) -&gt; Int {\n   case msg {\n     Decr -&gt; count - 1\n     Incr -&gt; count + 1\n   }\n }\n \n pub fn render(count: Int) -&gt; Element(CounterMsg) {\n   element.div([], [\n     element.button([event.on_click(Decr)], [element.text(&quot;-&quot;)]),\n     element.text(int.to_string(count)),\n     element.button([event.on_click(Incr)], [element.text(&quot;+&quot;)]),\n   ])\n }\n \n\n To integrate this into a large Lustre application, we need to do some plumbing.\n Here&#39;s how we might have a parent application that renders multiple counters:\n\n \n import counter.{Msg as CounterMsg}\n import gleam/list\n import gleam/map.{Map}\n import gleam/result\n import lustre\n import lustre/element.{Element}\n import lustre/event\n \n pub fn main() {\n   let app = lustre.simple(init(), update, render)\n   assert Ok(_) = lustre.start(app, &quot;#root&quot;)\n }\n \n type Model = Map(Int, Int)\n \n fn init() -&gt; Model {\n   map.from_list([\n     #(0, 0),\n     #(1, 0),\n     #(2, 0),\n   ])\n }\n \n type Msg {\n   UpdateCounter(Int, CounterMsg)\n }\n \n pub fn update(model: Model, msg: Msg) -&gt; Model {\n   case msg {\n     UpdateCounter(id, counter_msg) -&gt; {\n       map.get(model, id)\n       // Call the counter&#39;s update function with the `counter_msg`. This will\n       // either be `counter.Decr` or `counter.Incr`.\n       |&gt; result.map(counter.update(_, counter_msg))\n       |&gt; result.map(map.insert(model, id, _))\n       |&gt; result.unwrap(model)\n     }\n   }\n }\n \n pub fn render(model: Model) -&gt; Element(Msg) {\n   let counters = {\n     use els, id, count &lt;- map.fold(model, [])\n     let counter = counter.render(counr)\n \n     // Right now `counter` produces messages of type `counter.Msg`. Here we&#39;re\n     // calling `element.map` to wrap that message in our main app&#39;s `Msg` type.\n     [element.map(counter, UpdateCounter(id, _)), ...els]\n   }\n \n   element.div([], list.reverse(counters))\n }\n \n\n If this feels like a lt of work... sometimes it is! Take a look at the docs\n for [`stateful`](#stateful) elements to see how all this can be encapsulated.\n\nSynonyms:\nlustre/element.map\nlustre/element map","url":"lustre/element.html#map"},{"doc":"lustre/element","title":"map_","content":"pub fn map_(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.map_\nlustre/element map_","url":"lustre/element.html#map_"},{"doc":"lustre/element","title":"mark","content":"pub fn mark(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.mark\nlustre/element mark","url":"lustre/element.html#mark"},{"doc":"lustre/element","title":"mathml","content":"pub fn mathml(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.mathml\nlustre/element mathml","url":"lustre/element.html#mathml"},{"doc":"lustre/element","title":"menu","content":"pub fn menu(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.menu\nlustre/element menu","url":"lustre/element.html#menu"},{"doc":"lustre/element","title":"meta","content":"pub fn meta(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element.meta\nlustre/element meta","url":"lustre/element.html#meta"},{"doc":"lustre/element","title":"meter","content":"pub fn meter(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.meter\nlustre/element meter","url":"lustre/element.html#meter"},{"doc":"lustre/element","title":"nav","content":"pub fn nav(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.nav\nlustre/element nav","url":"lustre/element.html#nav"},{"doc":"lustre/element","title":"node","content":"pub external fn node(\n  tag: String,\n  attrs: List(Attribute(msg)),\n  children: List(Element(msg)),\n) -&gt; Element(msg)\n Construct a plain HTML element or registered Web Component by providing the\n tag name, a list of attributes (including event handlers), and a list of\n child elements.\n\nSynonyms:\nlustre/element.node\nlustre/element node","url":"lustre/element.html#node"},{"doc":"lustre/element","title":"noscript","content":"pub fn noscript(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.noscript\nlustre/element noscript","url":"lustre/element.html#noscript"},{"doc":"lustre/element","title":"object","content":"pub fn object(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.object\nlustre/element object","url":"lustre/element.html#object"},{"doc":"lustre/element","title":"ol","content":"pub fn ol(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.ol\nlustre/element ol","url":"lustre/element.html#ol"},{"doc":"lustre/element","title":"optgroup","content":"pub fn optgroup(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.optgroup\nlustre/element optgroup","url":"lustre/element.html#optgroup"},{"doc":"lustre/element","title":"option","content":"pub fn option(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.option\nlustre/element option","url":"lustre/element.html#option"},{"doc":"lustre/element","title":"output","content":"pub fn output(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.output\nlustre/element output","url":"lustre/element.html#output"},{"doc":"lustre/element","title":"p","content":"pub fn p(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.p\nlustre/element p","url":"lustre/element.html#p"},{"doc":"lustre/element","title":"param","content":"pub fn param(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.param\nlustre/element param","url":"lustre/element.html#param"},{"doc":"lustre/element","title":"picture","content":"pub fn picture(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.picture\nlustre/element picture","url":"lustre/element.html#picture"},{"doc":"lustre/element","title":"portal","content":"pub fn portal(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element.portal\nlustre/element portal","url":"lustre/element.html#portal"},{"doc":"lustre/element","title":"pre","content":"pub fn pre(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.pre\nlustre/element pre","url":"lustre/element.html#pre"},{"doc":"lustre/element","title":"progress","content":"pub fn progress(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.progress\nlustre/element progress","url":"lustre/element.html#progress"},{"doc":"lustre/element","title":"rp","content":"pub fn rp(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.rp\nlustre/element rp","url":"lustre/element.html#rp"},{"doc":"lustre/element","title":"rt","content":"pub fn rt(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.rt\nlustre/element rt","url":"lustre/element.html#rt"},{"doc":"lustre/element","title":"ruby","content":"pub fn ruby(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.ruby\nlustre/element ruby","url":"lustre/element.html#ruby"},{"doc":"lustre/element","title":"s","content":"pub fn s(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.s\nlustre/element s","url":"lustre/element.html#s"},{"doc":"lustre/element","title":"samp","content":"pub fn samp(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.samp\nlustre/element samp","url":"lustre/element.html#samp"},{"doc":"lustre/element","title":"section","content":"pub fn section(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.section\nlustre/element section","url":"lustre/element.html#section"},{"doc":"lustre/element","title":"select","content":"pub fn select(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.select\nlustre/element select","url":"lustre/element.html#select"},{"doc":"lustre/element","title":"slot","content":"pub fn slot(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.slot\nlustre/element slot","url":"lustre/element.html#slot"},{"doc":"lustre/element","title":"small","content":"pub fn small(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.small\nlustre/element small","url":"lustre/element.html#small"},{"doc":"lustre/element","title":"source","content":"pub fn source(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element.source\nlustre/element source","url":"lustre/element.html#source"},{"doc":"lustre/element","title":"span","content":"pub fn span(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.span\nlustre/element span","url":"lustre/element.html#span"},{"doc":"lustre/element","title":"stateful","content":"pub external fn stateful(\n  init: state,\n  render: fn(state, fn(state) -&gt; Nil) -&gt; Element(msg),\n) -&gt; Element(msg)\n A stateful element is exactly what it sounds like: an element with local\n encapsulated state! The `render` function we must provide is called with the\n element&#39;s current state as well as a function to set a new state. Whenever\n that function is called, the element is re-rendered.\n\n You might be wondering where the `stateless` version of this function is. \n Those are just regular Gleam functions that return `Element`s!\n\n We can implement a counter whose current value is stored locally like so:\n\n \n import gleam/int\n import lustre/attribute.{Attribute}\n import lustre/element.{Element}\n import lustre/event\n \n pub fn counter() -&gt; Element(msg) {\n   use state, set_state = lustre.stateful(0)\n \n   let decr = event.on(&quot;click&quot;, fn(_, _) { set_state(state - 1) })\n   let incr = event.on(&quot;click&quot;, fn(_, _) { set_state(state + 1) })\n \n   element.div([], [\n     element.button([decr], [element.text(&quot;-&quot;)]),\n     element.text(int.to_string(state)),\n     element.button([incr], [element.text(&quot;+&quot;)]),\n   ])\n }\n \n\n We can use this `counter` element anywhere in our app, we don&#39;t have to worry\n about passing state around or managing it in a parent component.\n\n When attaching event handlers to stateful elements, you&#39;ll typically have to\n use the generic `event.on` function rather than the more specific helpders.\n The helpers are designed to emit messages for your application, but stateful\n elements don&#39;t necessarily produce messages: calling `set_state` is a side\n effect.\n\nSynonyms:\nlustre/element.stateful\nlustre/element stateful","url":"lustre/element.html#stateful"},{"doc":"lustre/element","title":"strong","content":"pub fn strong(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.strong\nlustre/element strong","url":"lustre/element.html#strong"},{"doc":"lustre/element","title":"style","content":"pub fn style(attrs: List(Attribute(a)), css: String) -&gt; Element(a)\n\nSynonyms:\nlustre/element.style\nlustre/element style","url":"lustre/element.html#style"},{"doc":"lustre/element","title":"sub","content":"pub fn sub(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.sub\nlustre/element sub","url":"lustre/element.html#sub"},{"doc":"lustre/element","title":"summary","content":"pub fn summary(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.summary\nlustre/element summary","url":"lustre/element.html#summary"},{"doc":"lustre/element","title":"sup","content":"pub fn sup(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.sup\nlustre/element sup","url":"lustre/element.html#sup"},{"doc":"lustre/element","title":"svg","content":"pub fn svg(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.svg\nlustre/element svg","url":"lustre/element.html#svg"},{"doc":"lustre/element","title":"table","content":"pub fn table(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.table\nlustre/element table","url":"lustre/element.html#table"},{"doc":"lustre/element","title":"tbody","content":"pub fn tbody(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.tbody\nlustre/element tbody","url":"lustre/element.html#tbody"},{"doc":"lustre/element","title":"td","content":"pub fn td(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.td\nlustre/element td","url":"lustre/element.html#td"},{"doc":"lustre/element","title":"template","content":"pub fn template(attrs: List(Attribute(a)), children: List(\n    Element(a),\n  )) -&gt; Element(a)\n\nSynonyms:\nlustre/element.template\nlustre/element template","url":"lustre/element.html#template"},{"doc":"lustre/element","title":"text","content":"pub external fn text(content: String) -&gt; Element(msg)\n Render a Gleam string as an HTML text node.\n\nSynonyms:\nlustre/element.text\nlustre/element text","url":"lustre/element.html#text"},{"doc":"lustre/element","title":"textarea","content":"pub fn textarea(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element.textarea\nlustre/element textarea","url":"lustre/element.html#textarea"},{"doc":"lustre/element","title":"tfoot","content":"pub fn tfoot(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.tfoot\nlustre/element tfoot","url":"lustre/element.html#tfoot"},{"doc":"lustre/element","title":"th","content":"pub fn th(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.th\nlustre/element th","url":"lustre/element.html#th"},{"doc":"lustre/element","title":"thead","content":"pub fn thead(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.thead\nlustre/element thead","url":"lustre/element.html#thead"},{"doc":"lustre/element","title":"time","content":"pub fn time(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.time\nlustre/element time","url":"lustre/element.html#time"},{"doc":"lustre/element","title":"title","content":"pub fn title(attrs: List(Attribute(a)), name: String) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.title\nlustre/element title","url":"lustre/element.html#title"},{"doc":"lustre/element","title":"tr","content":"pub fn tr(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.tr\nlustre/element tr","url":"lustre/element.html#tr"},{"doc":"lustre/element","title":"track","content":"pub fn track(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.track\nlustre/element track","url":"lustre/element.html#track"},{"doc":"lustre/element","title":"u","content":"pub fn u(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.u\nlustre/element u","url":"lustre/element.html#u"},{"doc":"lustre/element","title":"ul","content":"pub fn ul(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.ul\nlustre/element ul","url":"lustre/element.html#ul"},{"doc":"lustre/element","title":"var_","content":"pub fn var_(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.var_\nlustre/element var_","url":"lustre/element.html#var_"},{"doc":"lustre/element","title":"video","content":"pub fn video(attrs: List(Attribute(a)), children: List(Element(a))) -&gt; Element(\n  a,\n)\n\nSynonyms:\nlustre/element.video\nlustre/element video","url":"lustre/element.html#video"},{"doc":"lustre/element","title":"wbr","content":"pub fn wbr(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element.wbr\nlustre/element wbr","url":"lustre/element.html#wbr"},{"doc":"lustre/element","title":"lustre/element","content":"","url":"lustre/element.html"},{"doc":"lustre/event","title":"checked","content":"pub fn checked(event: Dynamic) -&gt; Result(Bool, List(DecodeError))\n A helpful decoder to extract the `checked` property from an event triggered\n by a checkbox.\n\nSynonyms:\nlustre/event.checked\nlustre/event checked","url":"lustre/event.html#checked"},{"doc":"lustre/event","title":"mouse_position","content":"pub fn mouse_position(event: Dynamic) -&gt; Result(\n  #(Float, Float),\n  List(DecodeError),\n)\n A helpful decoder to grab the mouse&#39;s current x and y position in the\n viewport from an event object.\n\nSynonyms:\nlustre/event.mouse_position\nlustre/event mouse_position","url":"lustre/event.html#mouse_position"},{"doc":"lustre/event","title":"on","content":"pub fn on(name: String, handler: fn(Dynamic, fn(a) -&gt; Nil) -&gt; Nil) -&gt; Attribute(\n  a,\n)\n Attach custom event handlers to an element. A number of helper functions exist\n in this module to cover the most common events and use-cases, so you should\n check those out first.\n\n If you need to handle an event that isn&#39;t covered by the helper functions,\n then you can use `on` to attach a custom event handler. The callback receives\n a `Dynamic` representing the JavaScript event object, and a dispatch function\n you can use to send messages to the Lustre runtime.\n\n As a simple example, you can implement `on_click` like so:\n\n \n import lustre/attribute.{Attribute}\n import lustre/event\n \n pub fn on_click(msg: msg) -&gt; Attribute(msg) {\n   use _, dispatch &lt;- event.on(&quot;click&quot;)\n   dispatch(msg)\n }\n \n\n By using `gleam/dynamic` you can decode the event object and pull out all sorts\n of useful data. This is how `on_input` is implemented:\n\n \n import gleam/dynamic\n import lustre/attribute.{Attribute}\n import lustre/event\n \n pub fn on_input(msg: fn(String) -&gt; msg) -&gt; Attribute(msg) {\n   use event, dispatch &lt;- on(&quot;input&quot;)\n   let decode_value = dynamic.field(&quot;target&quot;, dynamic.field(&quot;value&quot;, dynamic.string))\n   let emit_value = fn(value) { dispatch(msg(value)) }\n \n   event\n   |&gt; decode_value\n   |&gt; result.map(emit_value)\n   |&gt; result.unwrap(Nil)\n }\n \n\n You can take a look at the MDN reference for events\n [here](https://developer.mozilla.org/en-US/docs/Web/API/Event) to see what\n you can decode. \n\n Unlike the helpers in the rest of this module, it is possible to simply ignore\n the dispatch function and not dispatch a message at all. In fact, we saw this\n with the `on_input` example above: if we can&#39;t decode the event object, we\n simply return `Nil` and do nothing.\n\n Beyond error handling, this can be used to perform side effects we don&#39;t need\n to observe in our main application loop, such as logging...\n\n \n import gleam/io\n import lustre/attribute.{Attribute}\n import lustre/event\n \n pub fn log_on_click(msg: String) -&gt; Attribute(msg) {\n   use _, _ &lt;- event.on(&quot;click&quot;)\n   io.println(msg)\n }\n \n\n ...or calling `set_state` from a `stateful` Lustre element:\n\n \n import gleam/int\n import lustre/attribute.{Attribute}\n import lustre/element.{Element}\n import lustre/event\n \n pub fn counter() -&gt; Element(msg) {\n   use state, set_state = lustre.stateful(0)\n \n   let decr = event.on(&quot;click&quot;, fn(_, _) { set_state(state - 1) })\n   let incr = event.on(&quot;click&quot;, fn(_, _) { set_state(state + 1) })\n \n   element.div([], [\n     element.button([decr], [element.text(&quot;-&quot;)]),\n     element.text(int.to_string(state)),\n     element.button([incr], [element.text(&quot;+&quot;)]),\n   ])\n }\n \n\nSynonyms:\nlustre/event.on\nlustre/event on","url":"lustre/event.html#on"},{"doc":"lustre/event","title":"on_blur","content":"pub fn on_blur(msg: a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_blur\nlustre/event on_blur","url":"lustre/event.html#on_blur"},{"doc":"lustre/event","title":"on_check","content":"pub fn on_check(msg: fn(Bool) -&gt; a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_check\nlustre/event on_check","url":"lustre/event.html#on_check"},{"doc":"lustre/event","title":"on_click","content":"pub fn on_click(msg: a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_click\nlustre/event on_click","url":"lustre/event.html#on_click"},{"doc":"lustre/event","title":"on_focus","content":"pub fn on_focus(msg: a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_focus\nlustre/event on_focus","url":"lustre/event.html#on_focus"},{"doc":"lustre/event","title":"on_input","content":"pub fn on_input(msg: fn(String) -&gt; a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_input\nlustre/event on_input","url":"lustre/event.html#on_input"},{"doc":"lustre/event","title":"on_keydown","content":"pub fn on_keydown(msg: fn(String) -&gt; a) -&gt; Attribute(a)\n Listens for key dow events on an element, and dispatches a message with the\n current key being pressed.\n\nSynonyms:\nlustre/event.on_keydown\nlustre/event on_keydown","url":"lustre/event.html#on_keydown"},{"doc":"lustre/event","title":"on_keypress","content":"pub fn on_keypress(msg: fn(String) -&gt; a) -&gt; Attribute(a)\n Listens for key presses on an element, and dispatches a message with the\n current key being pressed.\n\nSynonyms:\nlustre/event.on_keypress\nlustre/event on_keypress","url":"lustre/event.html#on_keypress"},{"doc":"lustre/event","title":"on_keyup","content":"pub fn on_keyup(msg: fn(String) -&gt; a) -&gt; Attribute(a)\n Listens for key up events on an element, and dispatches a message with the\n current key being released.\n\nSynonyms:\nlustre/event.on_keyup\nlustre/event on_keyup","url":"lustre/event.html#on_keyup"},{"doc":"lustre/event","title":"on_mouse_down","content":"pub fn on_mouse_down(msg: a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_mouse_down\nlustre/event on_mouse_down","url":"lustre/event.html#on_mouse_down"},{"doc":"lustre/event","title":"on_mouse_enter","content":"pub fn on_mouse_enter(msg: a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_mouse_enter\nlustre/event on_mouse_enter","url":"lustre/event.html#on_mouse_enter"},{"doc":"lustre/event","title":"on_mouse_leave","content":"pub fn on_mouse_leave(msg: a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_mouse_leave\nlustre/event on_mouse_leave","url":"lustre/event.html#on_mouse_leave"},{"doc":"lustre/event","title":"on_mouse_out","content":"pub fn on_mouse_out(msg: a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_mouse_out\nlustre/event on_mouse_out","url":"lustre/event.html#on_mouse_out"},{"doc":"lustre/event","title":"on_mouse_over","content":"pub fn on_mouse_over(msg: a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_mouse_over\nlustre/event on_mouse_over","url":"lustre/event.html#on_mouse_over"},{"doc":"lustre/event","title":"on_mouse_up","content":"pub fn on_mouse_up(msg: a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_mouse_up\nlustre/event on_mouse_up","url":"lustre/event.html#on_mouse_up"},{"doc":"lustre/event","title":"on_submit","content":"pub fn on_submit(msg: a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_submit\nlustre/event on_submit","url":"lustre/event.html#on_submit"},{"doc":"lustre/event","title":"value","content":"pub fn value(event: Dynamic) -&gt; Result(String, List(DecodeError))\n A helpful decoder to extract the `value` from an event object. This is handy\n for getting the value as a string from an input event, for example.\n\nSynonyms:\nlustre/event.value\nlustre/event value","url":"lustre/event.html#value"},{"doc":"lustre/event","title":"lustre/event","content":"","url":"lustre/event.html"},{"doc":"lustre/cmd","title":"Cmd","content":"pub opaque type Cmd(action)\n A `Cmd` represents some side effect we want the Lustre runtime to perform.\n It is parameterised by our app&#39;s `action` type because some effects need to\n get information back into your program.\n\n\nSynonyms:\nlustre/cmd.Cmd\nlustre/cmd Cmd","url":"lustre/cmd.html#Cmd"},{"doc":"lustre/cmd","title":"batch","content":"pub fn batch(cmds: List(Cmd(a))) -&gt; Cmd(a)\n \n\nSynonyms:\nlustre/cmd.batch\nlustre/cmd batch","url":"lustre/cmd.html#batch"},{"doc":"lustre/cmd","title":"from","content":"pub fn from(cmd: fn(fn(a) -&gt; Nil) -&gt; Nil) -&gt; Cmd(a)\n Create a `Cmd` from some custom side effect. This is mostly useful for \n package authors, or for integrating other libraries into your Lustre app.\n\n We pass in a function that recieves a `dispatch` callback that can be used\n to send messages to the Lustre runtime. We could, for example, create a `tick`\n command that uses the `setTimeout` JavaScript API to send a message to the\n runtime every second:\n\n \n import lustre/cmd.{Cmd}\n \n external fn set_interval(callback: fn() -&gt; any, interval: Int) =\n   &quot;&quot; &quot;window.setInterval&quot;\n \n pub fn every_second(msg: msg) -&gt; Cmd(msg) {\n   use dispatch &lt;- cmd.from\n \n   set_interval(fn() { dispatch(msg) }, 1000)\n }\n \n\nSynonyms:\nlustre/cmd.from\nlustre/cmd from","url":"lustre/cmd.html#from"},{"doc":"lustre/cmd","title":"map","content":"pub fn map(cmd: Cmd(a), f: fn(a) -&gt; b) -&gt; Cmd(b)\n\nSynonyms:\nlustre/cmd.map\nlustre/cmd map","url":"lustre/cmd.html#map"},{"doc":"lustre/cmd","title":"none","content":"pub fn none() -&gt; Cmd(a)\n Typically our app&#39;s `update` function needs to return a tuple of\n `#(model, Cmd(action))`. When we don&#39;t need to perform any side effects we\n can just return `none()`!\n\nSynonyms:\nlustre/cmd.none\nlustre/cmd none","url":"lustre/cmd.html#none"},{"doc":"lustre/cmd","title":"to_list","content":"pub fn to_list(cmd: Cmd(a)) -&gt; List(fn(fn(a) -&gt; Nil) -&gt; Nil)\n\nSynonyms:\nlustre/cmd.to_list\nlustre/cmd to_list","url":"lustre/cmd.html#to_list"},{"doc":"lustre/cmd","title":"lustre/cmd","content":"","url":"lustre/cmd.html"},{"doc":"lustre","title":"App","content":"pub opaque type App(model, msg)\n An `App` describes a Lustre application: what state it holds and what kind\n of actions get dispatched to update that model. The only useful thing you can\n do with an `App` is pass it to [`start`](#start).\n\n You can construct an `App` from the two constructors exposed in this module:\n [`basic`](#basic) and [`application`](#application). Although you can&#39;t do\n anything but [`start`](#start) them, the constructors are separated in case\n you want to set up an application but defer starting it until some later point\n in time.\n\n text\n                                       +--------+\n                                       |        |\n                                       | update |\n                                       |        |\n                                       +--------+\n                                         ^    |\n                                         |    | \n                                     Msg |    | #(Model, Cmd(Msg))\n                                         |    |\n                                         |    v\n +------+                      +------------------------+\n |      |  #(Model, Cmd(Msg))  |                        |\n | init |---------------------&gt;|     Lustre Runtime     |\n |      |                      |                        |\n +------+                      +------------------------+\n                                         ^    |\n                                         |    |\n                                     Msg |    | Model\n                                         |    |\n                                         |    v\n                                       +--------+\n                                       |        |\n                                       | render |\n                                       |        |\n                                       +--------+\n \n\n &lt;small&gt;Someone please PR the Gleam docs generator to fix the monospace font,\n thanks! 💖&lt;/small&gt;\n\n\nSynonyms:\nlustre.App\nlustre App","url":"lustre.html#App"},{"doc":"lustre","title":"Error","content":"pub type Error {\n  ElementNotFound\n}\n\nElementNotFound\n\n\nSynonyms:\nlustre.Error\nlustre Error","url":"lustre.html#Error"},{"doc":"lustre","title":"application","content":"pub fn application(init: #(a, Cmd(b)), update: fn(a, b) -&gt;\n    #(a, Cmd(b)), render: fn(a) -&gt; Element(b)) -&gt; App(a, b)\n An evolution of a [`simple`](#simple) app that allows you to return a\n [`Cmd`](./lustre/cmd.html#Cmd) from your `init` and `update`s. Commands give\n us a way to perform side effects like sending an HTTP request or running a\n timer and then dispatch actions back to the runtime to trigger an `update`.\n\n\n import lustre\n import lustre/cmd\n import lustre/element\n \n pub fn main () {\n   let init = #(0, tick())\n\n   let update = fn (model, msg) {\n     case msg {\n       Tick -&gt; #(model + 1, tick())\n     }\n   }\n\n   let render = fn (model) {\n     element.div([], [\n       element.text(&quot;Time elapsed: &quot;)\n       element.text(int.to_string(model))\n     ])\n   }\n   \n   let app = lustre.simple(init, update, render)\n   assert Ok(_) = lustre.start(app, &quot;#root&quot;)   \n }\n \n fn tick () -&gt; Cmd(Msg) {\n   cmd.from(fn (dispatch) {\n     setInterval(fn () {\n       dispatch(Tick)\n     }, 1000)\n   })\n }\n \n external fn set_timeout (f: fn () -&gt; a, delay: Int) -&gt; Nil \n   = &quot;&quot; &quot;window.setTimeout&quot;\n\nSynonyms:\nlustre.application\nlustre application","url":"lustre.html#application"},{"doc":"lustre","title":"element","content":"pub fn element(element: Element(a)) -&gt; App(Nil, a)\n Create a basic lustre app that just renders some element on the page.\n Note that this doesn&#39;t mean the content is static! With `element.stateful`\n you can still create components with local state.\n\n Basic lustre apps don&#39;t have any *global* application state and so the\n plumbing is a lot simpler. If you find yourself passing lots of state around,\n you might want to consider using [`simple`](#simple) or [`application`](#application)\n instead.\n\n \n import lustre\n import lustre/element\n \n pub fn main () {\n   let app = lustre.element(\n     element.h1([], [\n       element.text(&quot;Hello, world!&quot;) \n     ])\n   )\n \n   assert Ok(_) = lustre.start(app, &quot;#root&quot;)\n }\n \n\nSynonyms:\nlustre.element\nlustre element","url":"lustre.html#element"},{"doc":"lustre","title":"simple","content":"pub fn simple(init: a, update: fn(a, b) -&gt; a, render: fn(a) -&gt;\n    Element(b)) -&gt; App(a, b)\n If you start off with a simple `[element`](#element) app, you may find\n yourself leaning on [`stateful`](./lustrel/element.html#stateful) elements\n to manage model used throughout your app. If that&#39;s the case or if you know\n you need some global model from the get-go, you might want to construct a \n [`simple`](#simple) app instead.\n\n This is one app constructor that allows your HTML elements to dispatch actions\n to update your program model. \n\n \n import gleam/int\n import lustre\n import lustre/element\n import lustre/event\n \n type Msg {\n   Decr\n   Incr\n }\n \n pub fn main () {\n   let init = 0\n\n   let update = fn (model, msg) {\n     case msg {\n       Decr -&gt; model - 1\n       Incr -&gt; model + 1\n     }\n   }\n\n   let render = fn (model) {\n     element.div([], [\n       element.button([ event.on_click(Decr) ], [\n         element.text(&quot;-&quot;)\n       ]),\n\n       element.text(int.to_string(model)),\n\n       element.button([ event.on_click(Incr) ], [\n         element.text(&quot;+&quot;)\n       ])\n     ])\n   }\n \n   let app = lustre.simple(init, update, render)\n   assert Ok(_) = lustre.start(app, &quot;#root&quot;)\n }\n \n\nSynonyms:\nlustre.simple\nlustre simple","url":"lustre.html#simple"},{"doc":"lustre","title":"start","content":"pub fn start(app: App(a, b), selector: String) -&gt; Promise(\n  fn(b) -&gt; Nil,\n)\n Once you have created a app with either `basic` or `application`, you \n need to actually start it! This function will mount your app to the DOM\n node that matches the query selector you provide.\n\n If everything mounted OK, we&#39;ll get back a dispatch function that you can \n call to send actions to your app and trigger an update. \n\n\n import lustre\n\n pub fn main () {\n   let app = lustre.appliation(init, update, render)\n   assert Ok(dispatch) = lustre.start(app, &quot;#root&quot;)\n \n   dispatch(Incr)\n   dispatch(Incr)\n   dispatch(Incr)\n }\n\n\n This may not seem super useful at first, but by returning this dispatch\n function from your `main` (or elsewhere) you can get events into your Lustre\n app from the outside world.\n\nSynonyms:\nlustre.start\nlustre start","url":"lustre.html#start"},{"doc":"lustre","title":"lustre","content":" Lustre is a declarative framework for building Web apps in Gleam. ","url":"lustre.html"}]);